from pyqchem import get_output_from_qchem, Structure, QchemInput
from pyqchem.parsers.parser_frequencies import basic_frequencies
from posym import PointGroup, SymmetryModes
import matplotlib.pyplot as plt
import numpy as np


measures_m0 = []
measures_m1 = []
measures_m2 = []
energies = []
frequencies = []

scan_range = np.arange(-0.2, 0.21, 0.01)
for x_dist in scan_range:

    water = [[x_dist,           0.00000000e+00,  2.40297090e-01],
             [-1.43261539e+00, -1.75444785e-16, -9.61188362e-01],
             [1.43261539e+00,   1.75444785e-16, -9.61188362e-01]]
    water = np.array(water) * 0.529177249

    molecule_water = Structure(coordinates=water,
                               symbols=['O', 'H', 'H'],
                               charge=0,
                               multiplicity=1)

    qc_input = QchemInput(molecule_water,
                          jobtype='freq',
                          exchange='hf',
                          basis='6-31G',
                          sym_ignore=True,
                          )

    parsed_data, ee = get_output_from_qchem(qc_input, parser=basic_frequencies, read_fchk=True)

    molecule_coor = np.array(ee['structure'].get_coordinates())
    molecule_symbols = np.array(ee['structure'].get_symbols())

    # print(' structure')
    print('Final energy:', parsed_data['scf_energy'])
    energies.append(parsed_data['scf_energy'])
    modes = [np.array(m['displacement']) for m in parsed_data['modes']]
    freqs = [m['frequency'] for m in parsed_data['modes']]
    print('freqs: ', freqs)
    frequencies.append(freqs)

    sm = SymmetryModes(group='c2v', coordinates=molecule_coor, modes=modes, symbols=molecule_symbols)

    m = 1  # mode number
    measures_m0.append(sm.get_state_mode(0).get_ir_representation().values)
    measures_m1.append(sm.get_state_mode(1).get_ir_representation().values)
    measures_m2.append(sm.get_state_mode(2).get_ir_representation().values)

    print('sm', sm.get_state_mode(m))

    print(sm.get_measure_pos)
    for i in range(len(modes)):
        print('m {}:'.format(i + 1), sm.get_state_mode(i))
    print('----------------------\n')


pg = PointGroup(group='C2v')

plt.title('mode 1')
for ir, l in zip(np.array(measures_m0).T, pg.ir_labels):
    plt.plot(scan_range, ir, '-', label=l)
plt.legend()
plt.xlabel('Distortion (Bohr)')
plt.ylim(0, 1)

plt.figure()
plt.title('mode 2')
for ir, l in zip(np.array(measures_m1).T, pg.ir_labels):
    plt.plot(scan_range, ir, '-', label=l)
plt.legend()
plt.xlabel('Distortion (Bohr)')
plt.ylim(0, 1)

plt.figure()
plt.title('mode 3')
for ir, l in zip(np.array(measures_m2).T, pg.ir_labels):
    plt.plot(scan_range, ir, '-', label=l)
plt.legend()
plt.xlabel('Distortion (Bohr)')
plt.ylim(0, 1)

plt.figure()
plt.title('Frequencies')
for i, freq in enumerate(np.array(frequencies).T):
    plt.plot(scan_range, freq, '-', label='mode: {}'.format(i+1))
plt.legend()
plt.xlabel('Distortion (Bohr)')
plt.ylabel('Frequency (cm-1)')

plt.figure()
plt.title('energy')
plt.plot(scan_range, energies, '-')
plt.ylabel('Hartree')
plt.xlabel('Bohr')

plt.show()






# Simple single point calculation
from pyqchem import get_output_from_qchem, QchemInput, Structure
from pyqchem.parsers.basic import basic_parser_qchem
from posym import SymmetryFunction, SymmetryBase
import posym.algebra as al
import numpy as np
import matplotlib.pyplot as plt


molecule_cis = Structure(coordinates=[[ -1.07076839,   -2.13175980,    0.03234382],
                                      [ -0.53741536,   -3.05918866,    0.04995793],
                                      [ -2.14073783,   -2.12969357,    0.04016267],
                                      [ -0.39112115,   -0.95974916,    0.00012984],
                                      [  0.67884827,   -0.96181542,   -0.00769025],
                                      [ -1.15875076,    0.37505495,   -0.02522296],
                                      [ -0.62213437,    1.30041753,   -0.05065831],
                                      [ -2.51391203,    0.37767199,   -0.01531698],
                                      [ -3.04726506,    1.30510083,   -0.03293196],
                                      [ -3.05052841,   -0.54769055,    0.01011971]],
                         symbols=['C', 'H', 'H', 'C', 'H', 'C', 'H', 'C', 'H', 'H'])


# create qchem input
qc_input = QchemInput(molecule_cis,
                      jobtype='sp',
                      exchange='hf',
                      basis='sto-3g',
                      #sym_ignore=True
                      )

# calculate and parse qchem output
data_cis, ee_cis = get_output_from_qchem(qc_input,
                                         read_fchk=True,
                                         processors=4,
                                         parser=basic_parser_qchem)


molecule_trans = Structure(coordinates=[[ -1.06908233,   -2.13352097,   -0.00725330],
                                        [ -0.53502155,   -3.05996561,   -0.04439369],
                                        [ -2.13778918,   -2.13379901,    0.04533562],
                                        [ -0.39193053,   -0.95978774,   -0.02681816],
                                        [  0.67677629,   -0.95950970,   -0.07940766],
                                        [ -1.16057856,    0.37359983,    0.02664018],
                                        [ -2.22928521,    0.37332175,    0.07923299],
                                        [ -0.48342683,    1.54733308,    0.00707382],
                                        [ -1.01748758,    2.47377771,    0.04421474],
                                        [  0.58527987,    1.54761115,   -0.04551805]],
                           symbols=['C', 'H', 'H', 'C', 'H', 'C', 'H', 'C', 'H', 'H'])



# create qchem input
qc_input = QchemInput(molecule_trans,
                      jobtype='sp',
                      exchange='hf',
                      basis='sto-3g',
                      sym_ignore=True
                      )

# calculate and parse qchem output

data_trans, ee_trans = get_output_from_qchem(qc_input,
                                             read_fchk=True,
                                             processors=4,
                                             parser=basic_parser_qchem,
                                             )

coordinates_cis = ee_cis['structure'].get_coordinates()
coordinates_trans = ee_trans['structure'].get_coordinates()

coefficients_cis = ee_cis['coefficients']
coefficients_trans = ee_trans['coefficients']

basis = ee_trans['basis']

from posym.tools import get_basis_set, build_orbital

basis_set_trans = get_basis_set(coordinates_trans, basis)
basis_set_cis = get_basis_set(coordinates_cis, basis)


o1_cis = build_orbital(basis_set_cis, coefficients_cis['alpha'][15])
o1_trans = build_orbital(basis_set_trans, coefficients_trans['alpha'][15])

x = np.linspace(-5, 5, 50)
y = np.linspace(-5, 5, 50)

X, Y = np.meshgrid(x, y)

Z = o1_cis(X, Y, np.zeros_like(X))
plt.imshow(Z, interpolation='bilinear', origin='lower', cmap='seismic')
plt.figure()
plt.contour(X, Y, Z, colors='k')
plt.show()


Z = o1_trans(X, Y, np.zeros_like(X))
plt.imshow(Z, interpolation='bilinear', origin='lower', cmap='seismic')
plt.figure()
plt.contour(X, Y, Z, colors='k')
plt.show()


def get_wf_symm(orbitals_symm, alpha=(1,), beta=(1,)):
    total_sym = None
    for a, o in zip(alpha, orbitals_symm):
        if a == 1:
            total_sym = total_sym * o if total_sym is not None else o

    for b, o in zip(beta, orbitals_symm):
        if b == 1:
            total_sym = total_sym * o if total_sym is not None else o

    return total_sym


print('\nCIS\n----')
cis_orbitals_sym = []
for i, orbital_coeff in enumerate(coefficients_cis['alpha']):
    orbital = build_orbital(basis_set_cis, orbital_coeff)
    sym_orbital = SymmetryFunction('c2v', orbital)
    print('Symmetry O{}: '.format(i+1), sym_orbital)
    cis_orbitals_sym.append(sym_orbital)


cis_wf_0 = get_wf_symm(cis_orbitals_sym,
                       alpha=[1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0],
                       beta=[ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0])
cis_wf_1 = get_wf_symm(cis_orbitals_sym,
                       alpha=[1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0],
                       beta=[ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0])
cis_wf_2 = get_wf_symm(cis_orbitals_sym,
                       alpha=[1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0],
                       beta=[ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0])

# print(cis_wf_0, cis_wf_1, cis_wf_2)

cis_dm = SymmetryBase(group='C2v', rep='B1') + \
         SymmetryBase(group='C2v', rep='B2') + \
         SymmetryBase(group='C2v', rep='A1')

def check_transition(transtion):
    if al.dot(transtion, SymmetryBase(group='C2v', rep='A1')) > 0.1:
        return 'Allowed'
    else:
        return 'Forbidden'

print('wf_0 -> wf_1: ', cis_wf_0 * cis_dm * cis_wf_1, check_transition(cis_wf_0 * cis_dm * cis_wf_1))
print('wf_0 -> wf_2: ', cis_wf_0 * cis_dm * cis_wf_2, check_transition(cis_wf_0 * cis_dm * cis_wf_2))


print('\nTRANS\n----')
trans_orbitals_sym = []
for i, orbital_coeff in enumerate(coefficients_trans['alpha']):
    orbital = build_orbital(basis_set_trans, orbital_coeff)
    sym_orbital = SymmetryFunction('c2h', orbital)
    print('Symmetry O{}: '.format(i+1), sym_orbital)
    trans_orbitals_sym.append(sym_orbital)

trans_wf_0 = get_wf_symm(trans_orbitals_sym,
                         alpha=[1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0],
                         beta=[ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0])
trans_wf_1 = get_wf_symm(trans_orbitals_sym,
                         alpha=[1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0],
                         beta=[ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0])
trans_wf_2 = get_wf_symm(trans_orbitals_sym,
                         alpha=[1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0],
                         beta=[ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0])

# print(trans_wf_0, trans_wf_1, trans_wf_2)

trans_dm = SymmetryBase(group='C2h', rep='Bu') + \
           SymmetryBase(group='C2h', rep='Au')


def check_transition(transition):
    if al.dot(transition, SymmetryBase(group='C2h', rep='Ag')) > 0.1:
        return 'Allowed'
    else:
        return 'Forbidden'


print('wf_0 -> wf_1: ', trans_wf_0 * trans_dm * trans_wf_1, check_transition(trans_wf_0 * trans_dm * trans_wf_1))
print('wf_0 -> wf_2: ', trans_wf_0 * trans_dm * trans_wf_2, check_transition(trans_wf_0 * trans_dm * trans_wf_2))



from posym import PointGroup, SymmetryBase

pg = PointGroup(group='C2h')
print('\nTest Point group table')
print(pg)

bu = SymmetryBase(group='C2h', rep='Bu')
bg = SymmetryBase('C2h', 'Bg')
ag = SymmetryBase('C2h', 'Ag')

print('\nTest symmetry object operation')
print('bu * bg:', 2 * bu * bg)

print('(bu + bg) * (bu + bu + ag):', (bu + bg) * (bu + bu + ag))
state1 = (bu + bg) * (bu + bu + ag)

print('\nTest symmetry object representation')

pg = PointGroup(group='Td')
print(pg)

bg = SymmetryBase('C2h', 'Bg')
a1 = SymmetryBase(group='Td', rep='A1')
a2 = SymmetryBase(group='Td', rep='A2')
e = SymmetryBase(group='Td', rep='E')
t1 = SymmetryBase(group='Td', rep='T1')

print('e*e + a1:', e * e + 2 * a1)
print('t1*t1:', t1 * t1)

pg = PointGroup(group='C2v')
print('\nTest Point group table')
print(pg)

a1 = SymmetryBase(group='C2v', rep='A1')
b1 = SymmetryBase('C2v', 'B1')
a2 = SymmetryBase('C2v', 'A2')

print(a1 * a2 + b1)

from posym.algebra import dot, norm

print('\nTest dot product')
print('A1 . A1: ', dot(a1, a1))
print('E . E : ', dot(e, e))
print('T1 . E : ', dot(t1, e))
print('T1 . (T1 + E): ', dot(t1, t1 + e))
print('T1 . (T1 + E) [normalized]: ', dot(t1, t1 + e, normalize=True))
print('(T1 + E) . (T1 + E): ', dot(t1 + e, t1 + e))
print('norm((T1 + E)): ', norm(t1 + e))
print('(T1 + E) . (T1 + E) [normalized]: ', dot(t1 + e, t1 + e, normalize=True))
print('(0.6T1 + E) . T1 : ', dot(0.6*t1 + e, t1))
print('(0.5T1 + E) . T1  [normalized]: ', dot(t1, t1, normalize=True))



from pyqchem import get_output_from_qchem, Structure, QchemInput
from pyqchem.parsers.parser_frequencies import basic_frequencies
from pyqchem.parsers.parser_optimization import basic_optimization

from posym import SymmetryModes
from pyqchem.tools import get_geometry_from_pubchem
import numpy as np
import posym.algebra as al


sh6_coor = [[ 0.16290727, -0.36340852,  0.00000000],
            [ 1.47290727, -0.36340852,  0.00000000],
            [ 0.16290727,  0.94659148,  0.00000000],
            [ 0.16290727, -0.36340852,  1.31000000],
            [-1.14709273, -0.36340852,  0.00000000],
            [ 0.16290727, -1.67340852,  0.00000000],
            [ 0.16290727, -0.36340852, -1.31000000]]
sh6_sym = ['S', 'H', 'H', 'H', 'H', 'H', 'H']

sh6_mol = Structure(coordinates=sh6_coor,
                    symbols=sh6_sym,
                    charge=0,
                    multiplicity=1)

dicloro_coor = [[ 2.1437,  0.1015, -0.0002],
                [-2.1439, -0.1011, -0.0002],
                [ 0.5135, -0.4232,  0.0002],
                [-0.5132,  0.4227,  0.0002],
                [ 0.4242, -1.5014,  0.0001],
                [-0.4237,  1.5009,  0.0001]]
dcl_symbols = ['Cl', 'Cl', 'C', 'C', 'H', 'H']

# from scipy.spatial.transform import Rotation as R
# angles = [100, 200, 300]
# rotmol = R.from_euler('zyx', angles, degrees=True)
# dicloro_coor = rotmol.apply(dicloro_coor)

dichloro_mol = Structure(coordinates=dicloro_coor,
                         symbols=dcl_symbols,
                         charge=0,
                         multiplicity=1)

methane_mol = get_geometry_from_pubchem('methane')
ammonia_mol = get_geometry_from_pubchem('ammonia')
water_mol = get_geometry_from_pubchem('water')
h2o2_mol = get_geometry_from_pubchem('h2o2')


for molecule, group in zip([h2o2_mol, sh6_mol, water_mol, dichloro_mol, methane_mol, ammonia_mol],
                           ['c2',      'Oh',     'c2v',       'c2h',       'Td',        'c3v']):

    qc_input = QchemInput(molecule,
                          jobtype='opt',
                          exchange='hf',
                          basis='sto-3g',
                          #geom_opt_tol_gradient=1,
                          #geom_opt_tol_energy=1,
                          #geom_opt_max_cycles=500,
                          )

    parsed_data = get_output_from_qchem(qc_input, parser=basic_optimization, processors=6)

    qc_input = QchemInput(parsed_data['optimized_molecule'],
                          jobtype='freq',
                          exchange='hf',
                          basis='sto-3g',
                          #sym_ignore=True,
                          )

    print(molecule)
    parsed_data, ee = get_output_from_qchem(qc_input, parser=basic_frequencies,
                                            read_fchk=True, processors=6)

    molecule_coor = np.array(ee['structure'].get_coordinates())
    molecule_symbols = np.array(ee['structure'].get_symbols())

    # print(' structure')
    print('\nPoint Group: {}'.format(group))
    print('Final energy:', parsed_data['scf_energy'])

    modes = [np.array(m['displacement']) for m in parsed_data['modes']]
    freqs = [m['frequency'] for m in parsed_data['modes']]
    # print('freqs: ', freqs)
    # print(molecule_coor)

    sm = SymmetryModes(group=group, coordinates=molecule_coor, modes=modes, symbols=molecule_symbols)

    for i in range(len(modes)):
        print('m {:2}: {:8.3f} :'.format(i + 1, freqs[i]), sm.get_state_mode(i))
    print('Total: ', sm)
    print('Dot: ', al.dot(sm, sm))
    print('angles: ', sm.orientation_angles)



import numpy as np
from scipy.spatial.transform import Rotation as R
from symgroupy import Symgroupy
import matplotlib.pyplot as plt
from itertools import permutations


def get_perpendicular(vector):
    for i, v in enumerate(vector):
        if np.abs(v) < 1e-5:
            v1 = np.zeros_like(vector)
            v1[i] = 1
            v2 = np.cross(vector, v1).tolist()

            return v1, v2

    for i, v in enumerate(vector):
        if np.abs(v) >= 1e-5:

            vr = np.array(vector).copy()
            vr[i] = -vr[i]

            v1 = np.cross(vector, vr).tolist()
            v2 = np.cross(vector, v1).tolist()

            return v1, v2


def rotation(angle, rotation_axis):

    rotation_vector = angle * np.array(rotation_axis) / np.linalg.norm(rotation_axis)
    rotation = R.from_rotvec(rotation_vector)

    return rotation.as_matrix()


def prepare_vector(positions, vector):

    #return np.dot(positions.T, positions) - np.dot(vector.T, positions)
    return np.array(vector) + np.array(positions)


def reflection(reflection_axis):

    reflection_axis = np.array(reflection_axis) / np.linalg.norm(reflection_axis)

    axis1, axis2 = get_perpendicular(reflection_axis)

    return np.outer(axis1, axis1) + np.outer(axis2, axis2) - np.outer(reflection_axis, reflection_axis)


def inversion():
    return -np.identity(3)


water = [[ 0.00000000e+00,  0.00000000e+00,  2.40297090e-01],
         [-1.43261539e+00, -1.75444785e-16, -9.61188362e-01],
         [ 1.43261539e+00,  1.75444785e-16, -9.61188362e-01],
         #[1, 0, 0]
         ]
water = np.array(water)*0.5
symbols = ['O', 'H', 'H']

rotation_axis = [0, 0, 1]


from pyqchem import get_output_from_qchem, Structure, QchemInput
from pyqchem.parsers.parser_frequencies import basic_frequencies
import matplotlib.pyplot as plt
from pyqchem.errors import OutputError



molecule = Structure(coordinates=water,
                     symbols=symbols,
                     charge=0,
                     multiplicity=1)

qc_input = QchemInput(molecule,
                      jobtype='freq',
                      exchange='hf',
                      basis='sto-3g',
                      )

parsed_data, ee = get_output_from_qchem(qc_input, parser=basic_frequencies, read_fchk=True)

opt_molecule = ee['structure']

print(' structure')
water = np.array(opt_molecule.get_coordinates())
print('Final energy:', parsed_data['scf_energy'])

mode = np.array(parsed_data['modes'][2]['displacement'])
complete_modes = [np.array(m['displacement']) for m in parsed_data['modes']]

with open('test_rot.xyz', 'w') as f:
    for angle in np.arange(0, 2*np.pi, 0.1):

        rotated_water = np.dot(rotation(angle, rotation_axis), water.T).T

        f.write('{}\n\n'.format(len(rotated_water) + 1))
        for r, l in zip(rotated_water, symbols):
            f.write(l + ' {:5.2f} {:5.2f} {:5.2f}\n'.format(*r))

        v1 = water[1] - water[0]
        r1 = water[2]

        #print(prepare_vector(r1, v1))

        rotated_vector = np.dot(rotation(angle, rotation_axis), prepare_vector(r1, v1))
        rotated_position = rotated_vector + rotated_water[2]

        f.write('C' + ' {:5.2f} {:5.2f} {:5.2f}\n'.format(*rotated_position))
        # print(np.dot(rotated_vector, v1))

with open('test_ref.xyz', 'w') as f:

    print(reflection(rotation_axis))

    reflected_water = np.dot(reflection(rotation_axis), water.T).T

    for coor in [water, reflected_water]:
        f.write('{}\n\n'.format(len(coor)))
        for r, l  in zip(coor, symbols):
            f.write(l + ' {:5.2f} {:5.2f} {:5.2f}\n'.format(*r))


class Operation:
    def __init__(self, coordinates):
        self._coordinates = coordinates

        self._measure_mode = []
        self._measure_coor = []

    def get_permutation(self, operation):
        operated_coor = np.dot(operation, self._coordinates.T).T

        coor_list = []
        permu_list = []
        for iter in permutations(enumerate(operated_coor), len(operated_coor)):
            permu_coor = np.array([c[1] for c in iter])
            iter_num = [c[0] for c in iter]

            coor_list.append(np.average(np.linalg.norm(np.subtract(self._coordinates, permu_coor), axis=0)))
            permu_list.append(iter_num)

        return np.min(coor_list), permu_list[np.nanargmin(coor_list)]

    def get_measure(self):

        return np.array(self._measure_mode)
        # return np.average(self._measure_total_modes, axis=1)

    def get_coor_measure(self):
        #  normalization

        sum_list = []
        for r1 in self._coordinates:
            for r2 in self._coordinates:
                subs = np.subtract(r1, r2)
                sum_list.append(np.dot(subs, subs))
        d = np.average(sum_list)

        return np.average(self._measure_coor)/d


class Rotation(Operation):
    def __init__(self, coordinates, modes, axis, order):
        super().__init__(coordinates)

        self._axis = axis
        self._order = order

        for angle in np.arange(2*np.pi/order, 2*np.pi, 2*np.pi/order):
            operation = rotation(angle, rotation_axis)
            operated_coor = np.dot(operation, self._coordinates.T).T

            for mode in modes:

                operated_mode = np.dot(operation, prepare_vector(self._coordinates, mode).T).T - operated_coor
                norm_1 = np.linalg.norm(mode, axis=1)

                mesure_coor, permu  = self.get_permutation(operation)

                permu_mode = np.array(operated_mode)[permu]
                norm_2 = np.linalg.norm(permu_mode, axis=1)

                self._measure_mode.append(np.average(np.divide(np.diag(np.dot(mode, permu_mode.T)), norm_1 * norm_2)))
                self._measure_coor.append(mesure_coor)


class Rotation_old:
    def __init__(self, coordinates, modes, axis, order):
        self._axis = axis
        self._order = order
        self._coordinates = coordinates

        self._measure_total_modes = []
        for mode in modes:
            self._measure_mode = []
            self._measure_coor = []

            for angle in np.arange(2*np.pi/order, 2*np.pi, 2*np.pi/order):
                operation = rotation(angle, rotation_axis)

                operated_coor = np.dot(operation, self._coordinates.T).T
                operated_mode = np.dot(operation, prepare_vector(self._coordinates, mode).T).T - operated_coor
                norm_1 = np.linalg.norm(mode, axis=1)

                mode_list = []
                coor_list = []
                for iter_coor, iter_modes in zip(permutations(operated_coor, len(operated_coor)),
                                                 permutations(operated_mode, len(operated_coor))):

                    permu_coor = np.array(iter_coor).reshape(len(self._coordinates), -1)
                    permu_mode = np.array(iter_modes).reshape(len(self._coordinates), -1)

                    coor_list.append(np.average(np.linalg.norm(np.subtract(self._coordinates, permu_coor), axis=0)))

                    norm_2 = np.linalg.norm(permu_mode, axis=1)
                    mode_list.append(np.average(np.divide(np.diag(np.dot(mode, permu_mode.T)), norm_1 * norm_2)))

                self._measure_mode.append(mode_list[np.nanargmin(coor_list)])
                self._measure_coor.append(np.min(coor_list))

            self._measure_total_modes.append(self._measure_mode)

    def get_measure(self):
        print(self._measure_mode)
        return np.average(self._measure_total_modes, axis=1)

    def get_coor_measure(self):

        # normalization
        sum_list = []
        for r1 in self._coordinates:
            for r2 in self._coordinates:
                subs = np.subtract(r1, r2)
                sum_list.append(np.dot(subs, subs))
        d = np.average(sum_list)

        return np.average(self._measure_coor)/d


r = Rotation_old(water, complete_modes, rotation_axis, 2)
print('measure mode: ', r.get_measure())
print('measure coor', r.get_coor_measure())

print('---------')

r = Rotation(water, complete_modes, rotation_axis, 2)
print('measure mode: ', r.get_measure())
print('measure coor', r.get_coor_measure())

exit()

mode = complete_modes[2]

sum_list = []
for r1 in water:
    for r2 in water:
        subs = np.subtract(r1, r2)
        sum_list.append(np.dot(subs, subs))

d = np.average(sum_list)

measures = []
modes = []
for a in np.arange(0, 2*np.pi, 0.01):

    operation = rotation(a, rotation_axis)
    #operation = reflection([1, 0, 0])

    operated_water = np.dot(operation, water.T).T
    rotated_vector = np.dot(operation, prepare_vector(water, mode).T).T - operated_water
    print(rotated_vector)
    print('mode')
    print(mode)

    # print(mode[1], rotated_vector[0])
    norm = np.linalg.norm(mode, axis=1)
    #norm2 = np.linalg.norm(rotated_vector, axis=1)

    #mode_list = np.average(np.divide(np.diag(np.dot(mode, rotated_vector.T)), norm * norm2))
    #print('->', mode_list)

    #exit()
    mode_list = []
    measure_list = []
    for iter_water, iter_modes in zip(permutations(operated_water, len(operated_water)),
                                       permutations(rotated_vector, len(operated_water))):
        #print(iter_water)
        new_water = np.array(iter_water).reshape(len(water), -1)
        mode_water = np.array(iter_modes).reshape(len(water), -1)

        measure_list.append(np.average(np.linalg.norm(np.subtract(water, new_water), axis=0)))

        norm2 = np.linalg.norm(mode_water, axis=1)
        mode_list.append(np.average(np.divide(np.diag(np.dot(mode, mode_water.T)), norm * norm2)))
        #print('->', mode_list)

    #exit()
    #mode_list = np.multiply(mode_list, 1 - np.array(measure_list)/d)
    print('----')
    measures.append(np.min(measure_list)/d)
    modes.append(mode_list[np.nanargmin(measure_list)])
    #modes.append(np.max(mode_list))

overlap = (1 - np.array(measures))

#print(modes)
#exit()
plt.plot(np.arange(0, 2*np.pi, 0.01), measures, label='measure')
plt.plot(np.arange(0, 2*np.pi, 0.01), modes, label='modes')
#plt.plot(np.arange(0, 2*np.pi, 0.01), measure, label='measure')
plt.plot(np.arange(0, 2*np.pi, 0.01), overlap, '--', label='overlap')
plt.legend()

print(water)

a = Symgroupy(coordinates=water, group='ci')
print('csm', a.csm/100)

plt.show()



from setuptools import setup, Extension
import numpy

include_dirs_numpy = [numpy.get_include()]


def check_compiler():
    import subprocess
    output = subprocess.Popen(['gcc'], stderr=subprocess.PIPE).communicate()[1]
    if b'clang' in output:
        return 'clang'
    if b'gcc' in output:
        return 'gcc'


if check_compiler() == 'clang':
    integrals = Extension('posym.integrals',
                          extra_compile_args=['-std=c99'],
                          include_dirs=include_dirs_numpy,
                          sources=['c/integrals.c'])

else:
    print ('openmp is used')
    integrals = Extension('posym.integrals',
                          extra_compile_args=['-std=c99', '-fopenmp'],
                          extra_link_args=['-lgomp'],
                          include_dirs=include_dirs_numpy,
                          sources=['c/integrals.c'])

setup(name='posym',
      version=0.1,
      description='posym module',
      long_description=open('README.md').read(),
      long_description_content_type='text/markdown',
      author='Abel Carreras',
      url='https://github.com/abelcarreras/posym',
      author_email='abelcarreras83@gmail.com',
      packages=['posym',
                'posym.operations'],
      install_requires=['numpy', 'scipy', 'matplotlib'],
      license='MIT License',
      ext_modules=[integrals])
